=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-12-01 21:11:49 +0000
+++ bzrlib/branch.py	2005-12-01 21:49:24 +0000
@@ -236,7 +236,7 @@
     def set_root_id(self, file_id):
         raise NotImplementedError('set_root_id is abstract')
 
-    def print_file(self, file, revno):
+    def print_file(self, file, revision_id):
         """Print `file` to stdout."""
         raise NotImplementedError('print_file is abstract')
 
@@ -824,13 +824,22 @@
         return inv.root.file_id
 
     @needs_read_lock
-    def print_file(self, file, revno):
+    def print_file(self, file, revision_id):
         """See Branch.print_file."""
-        tree = self.revision_tree(self.get_rev_id(revno))
+        tree = self.revision_tree(revision_id)
         # use inventory as it was in that revision
         file_id = tree.inventory.path2id(file)
         if not file_id:
-            raise BzrError("%r is not present in revision %s" % (file, revno))
+            try:
+                revno = self.revision_id_to_revno(revision_id)
+            except errors.NoSuchRevision:
+                # TODO: This should not be BzrError,
+                # but NoSuchFile doesn't fit either
+                raise BzrError('%r is not present in revision %s' 
+                                % (file, revision_id))
+            else:
+                raise BzrError('%r is not present in revision %s'
+                                % (file, revno))
         tree.print_file(file_id)
 
     @needs_write_lock

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-12-01 21:11:49 +0000
+++ bzrlib/builtins.py	2005-12-01 21:49:24 +0000
@@ -1180,9 +1180,7 @@
 
     @display_command
     def run(self, filename, revision=None):
-        if revision is None:
-            raise BzrCommandError("bzr cat requires a revision number")
-        elif len(revision) != 1:
+        if revision is not None and len(revision) != 1:
             raise BzrCommandError("bzr cat --revision takes exactly one number")
         tree = None
         try:
@@ -1190,9 +1188,14 @@
             b = tree.branch
         except NotBranchError:
             pass
+
         if tree is None:
             b, relpath = Branch.open_containing(filename)
-        b.print_file(relpath, revision[0].in_history(b).revno)
+        if revision is None:
+            revision_id = b.last_revision()
+        else:
+            revision_id = revision[0].in_history(b).rev_id
+        b.print_file(relpath, revision_id)
 
 
 class cmd_local_time_offset(Command):

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-12-01 19:42:25 +0000
+++ bzrlib/osutils.py	2005-12-01 21:49:24 +0000
@@ -516,7 +516,6 @@
         return ''
 
 
-
 def terminal_width():
     """Return estimated terminal width."""
 

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2005-12-01 14:57:05 +0000
+++ bzrlib/tests/blackbox/__init__.py	2005-12-01 16:42:05 +0000
@@ -28,6 +28,7 @@
 
 def test_suite():
     testmod_names = [
+                     'bzrlib.tests.blackbox.test_cat',
                      'bzrlib.tests.blackbox.test_too_much',
                      'bzrlib.tests.blackbox.test_pull',
                      'bzrlib.tests.blackbox.test_versioning',

=== added file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2005-12-01 16:42:05 +0000
@@ -0,0 +1,54 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr cat.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+
+class TestCat(TestCaseInTempDir):
+
+    def test_cat(self):
+
+        def bzr(*args, **kwargs):
+            return self.run_bzr(*args, **kwargs)[0]
+
+        bzr('init')
+        open('a', 'wb').write('foo\n')
+        bzr('add', 'a')
+
+        # 'bzr cat' without an option should cat the last revision
+        bzr('cat', 'a', retcode=3)
+
+        bzr('commit', '-m', '1')
+        open('a', 'wb').write('baz\n')
+
+        self.assertEquals(bzr('cat', 'a'), 'foo\n')
+
+        bzr('commit', '-m', '2')
+        self.assertEquals(bzr('cat', 'a'), 'baz\n')
+        self.assertEquals(bzr('cat', 'a', '-r', '1'), 'foo\n')
+        self.assertEquals(bzr('cat', 'a', '-r', '-1'), 'baz\n')
+
+        rev_id = bzr('revision-history').strip().split('\n')[-1]
+
+        self.assertEquals(bzr('cat', 'a', '-r', 'revid:%s' % rev_id), 'baz\n')
+

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2005-12-01 16:35:08 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2005-12-01 21:49:24 +0000
@@ -16,31 +16,13 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-"""Black-box tests for bzr.
-
-These check that it behaves properly when it's invoked through the regular
-command-line interface. This doesn't actually run a new interpreter but 
-rather starts again from the run_bzr function.
+"""Black-box tests for bzr pull.
 """
 
-
-# XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
-# Note: Please don't add new tests here, it's too big and bulky.  Instead add
-# them into small suites in bzrlib.tests.blackbox.test_FOO for the particular
-# UI command/aspect that is being tested.
-
-
-from cStringIO import StringIO
 import os
-import re
-import shutil
 import sys
 
 from bzrlib.branch import Branch
-from bzrlib.clone import copy_branch
-from bzrlib.errors import BzrCommandError
-from bzrlib.osutils import has_symlinks
-from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
 from bzrlib.tests.blackbox import ExternalBase
 
 class TestPull(ExternalBase):

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2005-12-01 20:58:36 +0000
+++ bzrlib/transport/sftp.py	2005-12-01 21:49:24 +0000
@@ -37,7 +37,7 @@
 from bzrlib.trace import mutter, warning, error
 from bzrlib.transport import Transport, register_transport
 from bzrlib.osutils import pathjoin
-from bzrlib.ui import ui_factory
+import bzrlib.ui
 
 try:
     import paramiko
@@ -731,8 +731,9 @@
             #self._password = None
 
         # give up and ask for a password
-        password = ui_factory.get_password(prompt='SSH %(user)s@%(host)s password',
-                                           user=username, host=self._host)
+        password = bzrlib.ui.ui_factory.get_password(
+                prompt='SSH %(user)s@%(host)s password',
+                user=username, host=self._host)
         try:
             transport.auth_password(username, password)
         except paramiko.SSHException:
@@ -746,8 +747,9 @@
             transport.auth_publickey(username, key)
             return True
         except paramiko.PasswordRequiredException:
-            password = ui_factory.get_password(prompt='SSH %(filename)s password',
-                                               filename=filename)
+            password = bzrlib.ui.ui_factory.get_password(
+                    prompt='SSH %(filename)s password',
+                    filename=filename)
             try:
                 key = pkey_class.from_private_key_file(filename, password)
                 transport.auth_publickey(username, key)

